<?xml version="1.0" encoding="utf-8"?>
<!-- Copyright 1999-2017. Plesk International GmbH. All rights reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="
                        server/kav-settings
                        | cid[@type = 'sapp-scripts']
  "/>

  <!-- Transform maintenance mode to status if required. -->
  <xsl:template match="domain/properties/status/enabled | site/properties/status/enabled">
    <xsl:variable name="root" select="../../.."/>
    <xsl:choose>
      <xsl:when test="$root/phosting[@maintenance-mode='true']">
        <disabled-by name="client"/>
      </xsl:when>
      <xsl:otherwise>
        <enabled/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Create 'dom-param' element right after 'domain/properties/dns-zone' and 'site/properties/dns-zone' if it doesn't exist and create 'turnOffAction'/'selfTurnOffAction' params if required. -->
  <xsl:template match="domain/properties[not(dom-param)]/dns-zone | site/properties[not(dom-param)]/dns-zone">
    <xsl:variable name="root" select="../.."/>
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>

    <xsl:element name="dom-param">
      <xsl:element name="param">
        <xsl:element name="name">
          <xsl:text>seoRedirect</xsl:text>
        </xsl:element>
        <xsl:element name="value">
          <!-- none -->
          <xsl:text>bm9uZQ==</xsl:text>
        </xsl:element>
      </xsl:element>
      
      <xsl:if test="$root/phosting[@maintenance-mode='true'] and $root/properties/status/enabled">
        <xsl:element name="param">
            <xsl:element name="name">
                <xsl:text>turnOffAction</xsl:text>
            </xsl:element>
            <xsl:element name="value">
                <!-- suspend -->
                <xsl:text>c3VzcGVuZA==</xsl:text>
            </xsl:element>
        </xsl:element>
        <xsl:element name="param">
            <xsl:element name="name">
                <xsl:text>selfTurnOffAction</xsl:text>
            </xsl:element>
            <xsl:element name="value">
                <!-- suspend -->
                <xsl:text>c3VzcGVuZA==</xsl:text>
            </xsl:element>
        </xsl:element>
      </xsl:if>
    </xsl:element>

  </xsl:template>

  <!-- Create 'properties' element right after 'phosting/preferences' if it doesn't exist and copy 'ip'
       nodes from 'domain/properties'. -->
  <xsl:template match="domain/phosting[not(properties)]/preferences">
    <xsl:copy><xsl:apply-templates select="@*|node()"/></xsl:copy>

    <properties><xsl:copy-of select="ancestor::domain/properties/ip"/></properties>
  </xsl:template>

  <!-- Create 'properties' element right after '{s,f}hosting/url' if it doesn't exist and copy 'ip'
       nodes from 'domain/properties'. -->
  <xsl:template match="domain/*[(name()='shosting' or name()='fhosting') and not(properties)]/url">
    <xsl:copy><xsl:apply-templates select="@*|node()"/></xsl:copy>

    <properties><xsl:copy-of select="ancestor::domain/properties/ip"/></properties>
  </xsl:template>

  <!-- Copy 'ip' nodes from 'domain/properties' to 'domain/{p,s,f}hosting/properties' if there are no 'ip' nodes there. -->
  <xsl:template match="domain/*[name()='phosting' or name()='shosting' or name()='fhosting']/properties[not(ip)]">
    <xsl:copy>
      <xsl:copy-of select="ancestor::domain/properties/ip"/>
    </xsl:copy>
  </xsl:template>

  <!-- PfW before 11 uses 'oversell'/'strict' instead of 'true'/'false'. -->
  <xsl:template match="reseller/limits-and-permissions/limit[@name='oversell' and text() = 'oversell']">
	  <limit name="oversell">true</limit>
  </xsl:template>
  <xsl:template match="reseller/limits-and-permissions/limit[@name='oversell' and text() = 'strict']">
	  <limit name="oversell">false</limit>
  </xsl:template>

  <!-- Skip additional domains content. -->
  <xsl:template match="domain/phosting/sites/site/phosting/content/cid[@type='vhost' or @type='error_docs' or @type='private']">
  </xsl:template>

  <!-- Remove 'www' flag from domain node if there is 'www' subdomain, site or alias. -->
  <xsl:template match="domain[
	phosting/subdomains/subdomain[@name='www'] or
	preferences/domain-alias[@name=concat('www.', ancestor::domain/@name)] or
	phosting/sites/site[@name=concat('www.', ancestor::domain/@name)]
  ]/@www">
    <xsl:attribute name="www">false</xsl:attribute>
  </xsl:template>

  <!-- Remove admin password, because its restoration does not supported -->
  <xsl:template match="admin/preferences/admin-password"/>

  <!-- Correct wrong value of php_handler_type. -->
  <xsl:template match="phosting/limits-and-permissions/scripting/@php_handler_type | phosting/subdomains/subdomain/scripting/@php_handler_type">
    <xsl:choose>
      <xsl:when test=".='isapi' and count(/migration-dump/dump-info/os-description[@type = 'windows'])=0">
        <xsl:attribute name="php_handler_type">
          <xsl:text>module</xsl:text>
        </xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Add seo-redirect attribute. -->
  <xsl:template match="domain/preferences/domain-alias | site/preferences/domain-alias">
    <xsl:copy>
      <xsl:attribute name="seo-redirect">
        <xsl:text>false</xsl:text>
      </xsl:attribute>
      <xsl:apply-templates select="@*|node()" />
    </xsl:copy>
  </xsl:template>

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" />
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
