def get_main_security_file():
    """Get name of main .Security file of subscription

    :rtype: str | unicode
    """
    return u'.Security'


def list_webuser_security_files(subscription):
    """List all possible .Security files related to web users of specified subscription

    :type subscription: parallels.core.migrated_subscription.MigratedSubscription
    :rtype: list[str | unicode]
    """
    security_files = []

    for webuser in subscription.raw_dump.get_phosting_webuser_names():
        security_files.append(u".Web.%s.Security" % (webuser,))

    return security_files


def list_subdomain_security_files(subscription):
    """List all possible .Security files related to subdomains of specified subscription

    :type subscription: parallels.core.migrated_subscription.MigratedSubscription
    :rtype: list[str | unicode]
    """
    security_files = []

    for site in subscription.raw_dump.iter_sites():
        if site.parent_domain_name is not None:
            security_files.append(
                u'subdomains\\%s\\.Security' % (
                    site.short_name.encode('idna')
                )

            )

    return security_files
